using System;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;
using System.Globalization;

namespace ETTrans
{
	internal static class Utils
	{
		private static Icon _appIcon = null;
		public static Icon AppIcon
		{
			get
			{
				if (Utils._appIcon == null)
				{
					try
					{
						Utils._appIcon = Icon.ExtractAssociatedIcon(Assembly.GetExecutingAssembly().Location);
					}
					catch
					{
					}
				}
				return Utils._appIcon;
			}
		}
		public static string GetLanguageName(string languageId)
		{
			if (string.IsNullOrEmpty(languageId))
			{
				return "";
			}
			languageId = languageId.Replace('_', '-');
			try
			{
				CultureInfo ci = new CultureInfo(languageId);
				if (ci != null)
				{
					if ((!string.IsNullOrEmpty(ci.DisplayName)) && (!languageId.Replace('_', '-').Equals(ci.DisplayName)))
					{
						return ci.DisplayName;
					}
				}
			}
			catch
			{ }
			switch (languageId.ToLowerInvariant())
			{
				case "cs":
					return "Czech";
				case "de":
					return "German";
				case "en":
					return "English";
				case "en-ca":
					return "English (Canada)";
				case "en-gb":
					return "English (United Kingdom)";
				case "es":
					return "Spanish";
				case "fi":
					return "Finnish";
				case "ja":
					return "Japanese";
				case "fr":
					return "French";
				case "it":
					return "Italian";
				case "ko":
					return "Korean";
				case "nl":
					return "Dutch";
				case "pl":
					return "Polish";
				case "ru":
					return "Russian";
				case "sv":
					return "Swedish";
				case "tr":
					return "Turkish";
				case "zh-cn":
					return "Chinese (Simplified)";
				case "zh-tw":
					return "Chinese (Traditional)";
			}
			return languageId;
		}
	}
}
